@@ -32,6 +32,9 @@
  *	27-AUG-2002 Steve Lin for Discovery
  *	12-Dec-2002 Lineo Japan, Inc.
  *      12-Dec-2002 Sharp Corporation for Poodle and Corgi
+ *	16-Jan-2003 SHARP sleep_on -> interruptible_sleep_on
+ *      28-Nov-2003 Sharp Corporation for Tosa
+ *      26-Feb-2004 Lineo Solutions, Inc.  for Tosa
  *
  */
 
@@ -39,12 +42,17 @@
 /*
  *  Debug macros 
  */
+//#define DEBUG 1
 #ifdef DEBUG
 #  define DPRINTK(fmt, args...)	printk("%s: " fmt, __FUNCTION__ , ## args)
 #else
 #  define DPRINTK(fmt, args...)
 #endif
 
+//#define WUPSRC_DEBUG
+#ifdef WUPSRC_DEBUG
+unsigned int debug1,debug2,debug3,debug4;
+#endif
 
 #include <linux/config.h>
 #include <linux/module.h>
@@ -79,8 +87,15 @@
 #include <asm/arch/keyboard_poodle.h>
 #elif defined(CONFIG_ARCH_PXA_CORGI)
 #include <asm/arch/keyboard_corgi.h>
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+#include <asm/arch/keyboard_tosa.h>
+#include <linux/ac97_codec.h>
 #endif
 
+#ifdef LOGICAL_WAKEUP_SRC
+#include <asm/arch/sharpsl_wakeup.h>
+unsigned long logical_wakeup_src_mask = IDPM_WAKEUP_REC|IDPM_WAKEUP_SYNC;
+#endif
 #include "sharpsl_param.h"
 
 
@@ -113,8 +128,15 @@ extern void cpu_pxa_resume(void);	/* function the bootloader will call back */
 extern int cpu_pxa_do_suspend(void);
 extern unsigned short chkFatalBatt(void);
 extern int sharpsl_off_charge_battery(void);
-void pxa_ssp_init(void);
+extern int charge_status;
 
+#if defined(CONFIG_ARCH_PXA_POODLE)
+void pxa_ssp_init(void);
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+void tosa_ac97_init(void);
+int pxa_ac97_get(struct ac97_codec **codec, unsigned char *ac97_on);
+static unsigned char ac97_on = 0;
+#endif
 #if defined(CONFIG_ARCH_PXA_POODLE)
 #define PWER_RTC	0x80000000
 #define R_WAKEUP_SRC	(GPIO_bit(GPIO_AC_IN) | GPIO_bit(GPIO_CF_STSCHG) /*|
@@ -127,11 +149,41 @@ void pxa_ssp_init(void);
 			  PWER_RTC | GPIO_bit(GPIO_WAKEUP) | GPIO_bit(GPIO_GA_INT) )
 #elif defined(CONFIG_ARCH_PXA_CORGI)
 #define PWER_RTC	0x80000000
+#if defined(CONFIG_ARCH_PXA_SHEPHERD)
+#define R_WAKEUP_SRC	(GPIO_bit(GPIO_AC_IN) | GPIO_bit(GPIO_AK_INT) | GPIO_bit(GPIO_MAIN_BAT_LOW))
+#else
 #define R_WAKEUP_SRC	(GPIO_bit(GPIO_AC_IN) | GPIO_bit(GPIO_AK_INT))
+#endif
 #define F_WAKEUP_SRC	(GPIO_bit(GPIO_KEY_INT) | GPIO_bit(GPIO_WAKEUP) | \
 			 GPIO_bit(GPIO_AC_IN) | GPIO_bit(GPIO_MAIN_BAT_LOW))
 #define WAKEUP_SRC	( R_WAKEUP_SRC | F_WAKEUP_SRC | PWER_RTC )
 #define WAKEUP_DEF_SRC	( WAKEUP_SRC );
+
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+/*
+<F_WAKEUP_SRC>
+GPIO_POWERON           (0)
+GPIO_AC_IN             (2)
+GPIO_RECORD_BTN        (3)
+GPIO_SYNC              (4)
+GPIO_USB_IN            (5)
+GPIO_JACKET_DETECT     (7)
+GPIO_nSD_DETECT        (9)
+GPIO_nSD_INT           (10)
+GPIO_BAT1_CRG          (12)
+GPIO_CF_CD             (13)
+GPIO_BAT0_CRG          (14)
+*/
+#define PWER_RTC	0x80000000
+#define R_WAKEUP_SRC (GPIO_bit(GPIO_AC_IN) | GPIO_bit(GPIO_JACKET_DETECT))
+#define F_WAKEUP_SRC	(GPIO_bit(GPIO_POWERON) | GPIO_bit(GPIO_RESET) | \
+			 GPIO_bit(GPIO_AC_IN) | GPIO_bit(GPIO_RECORD_BTN) | \
+			 GPIO_bit(GPIO_SYNC) | GPIO_bit(GPIO_USB_IN) | \
+			 GPIO_bit(GPIO_JACKET_DETECT) )
+#define WAKEUP_SRC	( R_WAKEUP_SRC | F_WAKEUP_SRC | PWER_RTC )
+#define WAKEUP_DEF_SRC	( GPIO_bit(GPIO_POWERON) | GPIO_bit(GPIO_RESET) | \
+			  GPIO_bit(GPIO_AC_IN) | GPIO_bit(GPIO_JACKET_DETECT) | \
+			  GPIO_bit(GPIO_RECORD_BTN) | GPIO_bit(GPIO_SYNC) )
 #endif
 
 #if defined(CONFIG_SABINAL_DISCOVERY)
@@ -151,17 +203,63 @@ extern void sharpsl_battery_charge_hook(int mode);
 extern int corgi_wakeup_remocon_hook(void);
 #endif
 
-#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
 extern unsigned long cccr_reg;
 static int sharpsl_alarm_flag;
 #endif
 
+//@@#if defined(CONFIG_ARCH_PXA_TOSA)
+//@@extern int sharpsl_get_jacket_status(void);
+//@@#endif
+
 void PrintParamTable(void);
 
 #ifdef CONFIG_SABINAL_DISCOVERY
 static u32 alarm_enable=0;
 #endif
 
+#if defined(CONFIG_ARCH_PXA_SHEPHERD) || defined(CONFIG_ARCH_PXA_TOSA)
+int sharpsl_restart(void)
+{
+	return sharpsl_restart_core(0);
+}
+
+int sharpsl_restart_nonstop(void)
+{
+	return sharpsl_restart_core(1);
+}
+
+int sharpsl_restart_core(int nonstop_flag)
+{
+	int flag = 1;
+#if !defined(CONFIG_ARCH_PXA_TOSA)
+	if (nonstop_flag) {
+		SCP_REG_GPWR |= SCP_LED_GREEN;
+	}
+	else {
+		SCP_REG_GPWR &= ~SCP_LED_GREEN;
+	}
+#endif
+	RCSR = 0xf;
+
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	if( nonstop_flag && ((MSC0 & 0xffff0000) == 0x7ff00000) )
+	    MSC0 = (MSC0 & 0xffff)|0x7ee00000;
+ 						// GPIO reset
+ 	set_GPIO_mode(GPIO_ON_RESET | GPIO_OUT);
+ 	GPCR(GPIO_ON_RESET) |= GPIO_bit(GPIO_ON_RESET);
+#else
+	OSMR3 = OSCR+0x100;
+	OWER = 0x01;
+	OIER |= 0x08;
+
+	while(1) {
+		if ( flag++ > 0x20000000 ) break;
+	}
+#endif
+	return 0;
+}
+#else
 int sharpsl_restart(void)
 {
 	int flag = 1;
@@ -178,7 +276,7 @@ int sharpsl_restart(void)
 
 	return 0;
 }
-
+#endif
 
 #ifdef CONFIG_SABINAL_DISCOVERY
 static int discovery_wakeup_hook(void)
@@ -216,17 +314,47 @@ static int discovery_wakeup_hook(void)
 
 #if !defined(CONFIG_SABINAL_DISCOVERY)
 
-#if defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
 int sharpsl_wakeup_check_charge(void)
 {
+#if defined(CONFIG_ARCH_PXA_TOSA)
+  unsigned int pfer;
+  unsigned int prer;
+
+  pfer = F_WAKEUP_SRC & apm_wakeup_src_mask & WAKEUP_SRC;
+  prer = R_WAKEUP_SRC & apm_wakeup_src_mask & WAKEUP_SRC;
+  //printk("pfer = %08x",pfer);
+  //printk("prer = %08x",prer);
+
+  pfer &= ~(F_WAKEUP_SRC & R_WAKEUP_SRC);
+  prer &= ~(F_WAKEUP_SRC & R_WAKEUP_SRC);
+  //printk("pfer = %08x",pfer);
+  //printk("prer = %08x",prer);
+
+  pfer = ~GPLR0 & pfer;
+  prer = GPLR0 & prer;
+  //printk("pfer = %08x",pfer);
+  //printk("prer = %08x",prer);
+
+  if(pfer != 0 || prer != 0){
+    apm_wakeup_factor = pfer | prer;
+    return 0; // wakeup
+  }
+
+  if (  ( ( RTAR - RCNR ) <  20 ) && ( RTSR & RTSR_ALE )  ) {
+    return -1; // go off.
+  }
+
+  return 1; // continue.
+#else
   unsigned int temp;
 
   temp = ~GPLR0 & ( GPIO_bit(GPIO_AC_IN) | GPIO_bit(GPIO_KEY_INT) | GPIO_bit(GPIO_WAKEUP) );
   if ( temp != 0 ) {
     apm_wakeup_factor = temp;
   }
-
   return temp;
+#endif
 }
 #endif
 
@@ -234,31 +362,71 @@ static int sharpsl_wakeup_check(void)
 {
 	int i;
 	u32 gplr;
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	int batt_fault = ((PSSR & 0x02) != 0);
+#endif
 
 	/* setting GPIO */
 	GPDR0 = sys_ctx.gpdr0;
 	GAFR0_L = sys_ctx.gafr0_l;
 	GPDR0 &= ~WAKEUP_SRC;
-	GAFR0_L &= ~WAKEUP_SRC;
+//	GAFR0_L &= ~WAKEUP_SRC;
 	gplr = GPLR0;
 
+#ifdef WUPSRC_DEBUG
+	debug1 = PEDR;
+	debug2 = WAKEUP_SRC;
+	debug3 = apm_wakeup_src_mask;
+	debug4 = PSSR;
+#endif
 	apm_wakeup_factor = PEDR & WAKEUP_SRC & apm_wakeup_src_mask;
-#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
 	apm_wakeup_factor &= ~0x80000000;		/* clear ALARM */
 	if ( ( RTSR & 0x1 ) && ( RTSR & RTSR_ALE ) )
 #else
 	if ( RTSR & 0x1 )
 #endif
 		apm_wakeup_factor |= 0x80000000;		/* ALARM */
+
+
 	PEDR = WAKEUP_SRC;
 
-	if ( !apm_wakeup_factor )
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	if(batt_fault) PSSR = 0x02; /* clear BFS bit */
+#endif
+
+	if ( !apm_wakeup_factor ){
 		return 0;			/* no wakeup factor */
+	}
 
 #if defined(CONFIG_ARCH_PXA_CORGI)
 	gplr &= ~GPIO_bit(GPIO_KEY_INT);
 #endif
 
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	//if(PSSR & 0x02){
+	if(batt_fault){
+	  // Asserted battery fault.
+	  // It must have changed the main battery.
+	  if(GPIO_bit(GPIO_POWERON) & apm_wakeup_factor){
+	    int change_ac_status = 0;
+	    if ( apm_wakeup_src_mask & GPIO_bit(GPIO_AC_IN)){
+	      if ( !charge_status && ( gplr & GPIO_bit(GPIO_AC_IN) ) == 0)
+		change_ac_status = 1;
+
+	      if ( charge_status && ( gplr & GPIO_bit(GPIO_AC_IN) ) != 0)
+		change_ac_status = 1;
+	    }
+	    if(change_ac_status)
+	      apm_wakeup_factor |= GPIO_bit(GPIO_AC_IN);
+
+	    return apm_wakeup_factor;
+	  }
+	  if(GPIO_bit(GPIO_RESET) & apm_wakeup_factor){
+	    return apm_wakeup_factor;
+	  }
+	}
+#endif
 	/* Faulty operation check */
 	for (i = 0; i <= 15; i++) {
 #if defined(CONFIG_ARCH_PXA_CORGI)
@@ -287,33 +455,88 @@ static int sharpsl_wakeup_hook(void)
 	u32 gplr = GPLR0;
 	int is_resume = 0;
 
+	DPRINTK("GPLR0 = %x\n",gplr);
+#ifdef WUPSRC_DEBUG
+	printk("PEDR=%08x\n",debug1);
+	printk("src=%08x\n",debug2);
+	printk("src_mask=%08x\n",debug3);
+	printk("PSSR=%08x->%08x\n",debug4,PSSR);
+#endif
 	if ( (apm_wakeup_factor & GPIO_bit(GPIO_AC_IN))  &&
 	     sharpsl_battery_charge_hook ) {
+
+#if defined(CONFIG_ARCH_PXA_TOSA)
+		if ( !(gplr & GPIO_bit(GPIO_AC_IN)) ) {
+#else
 		if ( gplr & GPIO_bit(GPIO_AC_IN) ) {
+#endif
 			sharpsl_battery_charge_hook(2);	/* charge on */
 		} else 
+#if defined(CONFIG_ARCH_PXA_TOSA)
+		if ( gplr & GPIO_bit(GPIO_AC_IN) ) {
+#else
 		if ( !( gplr & GPIO_bit(GPIO_AC_IN) ) ) {
+#endif
 			sharpsl_battery_charge_hook(1);	/* charge off */
 		}
 	}
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	if ( (apm_wakeup_factor & GPIO_bit(GPIO_BAT0_CRG)) &&
+	     sharpsl_battery_charge_hook ) {
+		sharpsl_battery_charge_hook(0);  /* tosa: main battery full */
+	}
+	if ( (apm_wakeup_factor & GPIO_bit(GPIO_BAT1_CRG)) &&
+	     sharpsl_battery_charge_hook ) {
+		sharpsl_battery_charge_hook(3);  /* tosa: jacket battery full */
+	}
+#else
 	if ( (apm_wakeup_factor & GPIO_bit(GPIO_CHRG_FULL)) &&
 	     sharpsl_battery_charge_hook ) {
 		sharpsl_battery_charge_hook(0);  /* charge off */
 	}
+#endif
+
 #if defined(CONFIG_ARCH_PXA_POODLE)
 	if ( apm_wakeup_factor & GPIO_bit(GPIO_ON_KEY) )
 		is_resume |= GPIO_bit(GPIO_ON_KEY);
 #endif
 #if defined(CONFIG_ARCH_PXA_CORGI)
 	if ( apm_wakeup_factor & GPIO_bit(GPIO_KEY_INT) ) {
-		if (sharppda_kbd_is_wakeup())
+		if (sharppda_kbd_is_wakeup()){
 			is_resume |= GPIO_bit(GPIO_KEY_INT);
+		}
 	}
+#if defined(CONFIG_ARCH_PXA_SHEPHERD)
+	if ((apm_wakeup_factor & GPIO_bit(GPIO_MAIN_BAT_LOW)) &&
+		(GPLR(GPIO_MAIN_BAT_LOW) & GPIO_bit(GPIO_MAIN_BAT_LOW)) &&
+		sharppda_kbd_resetcheck()) {
+		/* apm_wakeup_src_mask = 0; */
+		sharpsl_off_mode = 1;
+		is_resume |= GPIO_bit(GPIO_MAIN_BAT_LOW);
+	}
+
+	// lock
+	if ((apm_wakeup_factor & GPIO_bit(GPIO_MAIN_BAT_LOW)) &&
+	    (GPLR(GPIO_MAIN_BAT_LOW) & GPIO_bit(GPIO_MAIN_BAT_LOW)) &&
+	    ( sharpsl_battery_charge_hook )) {
+	  sharpsl_battery_charge_hook(2);	/* charge on */
+	}
+
+	// unlock
+	if ((apm_wakeup_factor & GPIO_bit(GPIO_MAIN_BAT_LOW)) &&
+		!(GPLR(GPIO_MAIN_BAT_LOW) & GPIO_bit(GPIO_MAIN_BAT_LOW)) &&
+	    ( sharpsl_battery_charge_hook )) {
+	  sharpsl_battery_charge_hook(1);	/* charge off */
+	}
+#else
 	if ( apm_wakeup_factor & GPIO_bit(GPIO_MAIN_BAT_LOW) )
 		apm_wakeup_src_mask = 0;
 #endif
+#endif
+#if !defined(CONFIG_ARCH_PXA_TOSA)
 	if ( apm_wakeup_factor & GPIO_bit(GPIO_WAKEUP) )
 		is_resume |= GPIO_bit(GPIO_WAKEUP);
+#endif
 #if defined(CONFIG_ARCH_PXA_POODLE)
 	if ( (apm_wakeup_factor & GPIO_bit(GPIO_GA_INT)) && (LCM_KIC & 1) ) {
 		LCM_KIC &= ~0x100;
@@ -330,28 +553,116 @@ static int sharpsl_wakeup_hook(void)
 #endif
 #if defined(CONFIG_ARCH_PXA_CORGI)
 	if ( apm_wakeup_factor & GPIO_bit(GPIO_AK_INT) ) {
-		if ( corgi_wakeup_remocon_hook() )
+		if ( corgi_wakeup_remocon_hook() ){
 			is_resume |= GPIO_bit(GPIO_AK_INT);
+		}
 	}
 #endif
 
-	DPRINTK("alarm flag = %8x\n",sharpsl_alarm_flag);
-	if ( ( apm_wakeup_factor & PWER_RTC ) && !sharpsl_alarm_flag)
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	if ( (apm_wakeup_factor & GPIO_bit(GPIO_AC_IN)) ) {
+#ifdef LOGICAL_WAKEUP_SRC
+		if (logical_wakeup_src_mask&IDPM_WAKEUP_AC) {
+			is_resume |= GPIO_bit(GPIO_AC_IN);
+		}
+#endif
+	}
+	if (apm_wakeup_factor & GPIO_bit(GPIO_POWERON)) { // function key
+		if (sharppda_kbd_is_wakeup()){
+			is_resume |= GPIO_bit(GPIO_POWERON);
+		}
+	}
+	if (apm_wakeup_factor & GPIO_bit(GPIO_RECORD_BTN)) { // rec key
+		if (sharppda_kbd_is_wakeup()){
+			is_resume |= GPIO_bit(GPIO_RECORD_BTN);
+		}
+	}
+	if (apm_wakeup_factor & GPIO_bit(GPIO_SYNC)) { // sync key
+		if (sharppda_kbd_is_wakeup()){
+			is_resume |= GPIO_bit(GPIO_SYNC);
+		}
+	}
+	if (apm_wakeup_factor & GPIO_bit(GPIO_USB_IN)) {
+#ifdef LOGICAL_WAKEUP_SRC
+		if (logical_wakeup_src_mask&IDPM_WAKEUP_USBD) {
+			is_resume |= GPIO_bit(GPIO_USB_IN);
+		}
+#endif
+	}
+	if (apm_wakeup_factor & GPIO_bit(GPIO_JACKET_DETECT)){
+	        sharpsl_battery_charge_hook(4);
+#ifdef LOGICAL_WAKEUP_SRC
+		if (logical_wakeup_src_mask&IDPM_WAKEUP_JACKET) {
+			is_resume |= GPIO_bit(GPIO_JACKET_DETECT);
+		}
+#endif
+	}
+#endif
+
+	if ( ( apm_wakeup_factor & PWER_RTC ) && !sharpsl_alarm_flag){
 		is_resume |= PWER_RTC;
+	}
 
 	return is_resume;
 }
 #endif
 
-
+#if defined(CONFIG_ARCH_PXA_TOSA)
+#define BATTERY_CHECK_TIME	(60*5)	// 5 min
+#else
 #define BATTERY_CHECK_TIME	60*10	// 10 min
-extern int charge_status;
+#endif
+
+//extern int charge_status;
 extern int sharpsl_off_charge;
 
 
+#if defined(CONFIG_ARCH_PXA_TOSA)
+#if 0 /////////////////////////////////////////////////////////
+static void tc6393_susx(void)
+{
+#if 1
+  reset_scoop_jc_gpio(SCP_JC_TC3693_L3V_ON);
+//  reset_scoop_jc_gpio(SCP_JC_TC6393_SUSPEND);
+//  reset_scoop_gpio(SCP_TC6393_REST_IN);
+#endif
+
+//  TC6393_SYS_REG(TC6393_SYS_GPOOECR1) = TC6393_CARD_VCC_ON;
+//  TC6393_SYS_REG(TC6393_SYS_GPODSR1) = 0; /* CARD_VCC_OFF */
+#if 1
+//  TC6393_SYS_REG(TC6393_SYS_GPOOECR1) = TC6393_GPO_OE;
+//  TC6393_SYS_REG(TC6393_SYS_GPOOECR1) = TC6393_CARD_VCC_ON;
+  TC6393_SYS_REG(TC6393_SYS_GPODSR1) |= TC6393_CARD_VCC_ON;
+  TC6393_SYS_REG(TC6393_SYS_GPOOECR1) = TC6393_GPO_OE;
+#endif
+}
+
+static void tc6393_resx(void)
+{
+  set_GPIO_mode(GPIO11_3_6MHz_MD);
+  set_GPIO_mode(GPIO18_RDY_MD);
+  mdelay(1);
+  set_scoop_jc_gpio(SCP_JC_TC6393_SUSPEND);
+  mdelay(10);
+  set_scoop_gpio(SCP_TC6393_REST_IN);
+  //set_scoop_jc_gpio(SCP_JC_TC3693_L3V_ON);
+  TC6393_SYS_REG(TC6393_SYS_FER) = 0;
+												/* clock setting */
+  TC6393_SYS_REG(TC6393_SYS_PLL2CR) = 0x0cc1;
+  TC6393_SYS_REG(TC6393_SYS_CCR) = 0x1310;
+  TC6393_SYS_REG(TC6393_SYS_MCR) = 0x80AA;
+												/* GPIO */
+  TC6393_SYS_REG(TC6393_SYS_GPER) = 0x0030;		/* 15-0 GPO */
+  TC6393_SYS_REG(TC6393_SYS_GPOOECR1) = TC6393_GPO_OE;
+												/* 15-0 GPO set H */
+  TC6393_SYS_REG(TC6393_SYS_GPODSR1) = TC6393_CARD_VCC_ON;
+}
+#endif ///////////////////////////////////////////////
+#endif
 
 int pxa_suspend(void)
 {
+
 	unsigned long   flags;
 #if defined (CONFIG_SABINAL_DISCOVERY)
 	unsigned long	RTAR_buffer;
@@ -359,10 +670,11 @@ int pxa_suspend(void)
 #else
 	unsigned long RTAR_buffer;
 	unsigned long RTAR_buffer2;
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	unsigned long RCNR_buffer;
+#endif
 #endif
 
-
-	
 #ifndef CONFIG_SABINAL_DISCOVERY
 	sharpsl_off_state = 1;
 
@@ -392,6 +704,13 @@ int pxa_suspend(void)
 		cpu_xscale_sl_change_speed_145_without_lcd();
 	    cccr_reg = CCCR;
 	    break;
+#if defined(CONFIG_ARCH_PXA_SHEPHERD) || defined(CONFIG_ARCH_PXA_TOSA)
+	  case 0x161:
+	    if ( CCCR == 0x0161 ) break;
+		cpu_xscale_sl_change_speed_161();
+	    cccr_reg = CCCR;
+	    break;
+#endif
 	  }
 	  sharpsl_chg_freq &= 0x0000ffff;
 	  mdelay(500);
@@ -399,6 +718,10 @@ int pxa_suspend(void)
 	}
 #endif
 
+#if defined(CONFIG_ARCH_PXA_SHEPHERD) || defined(CONFIG_ARCH_PXA_TOSA)
+	if (sharpsl_off_mode == 2) sharpsl_restart();	/* off in maintenance */
+#endif
+
 	/* set up pointer to sleep parameters */
 	sleep_param = kmalloc (SLEEPDATA_SIZE*sizeof(long), GFP_ATOMIC);
 	if (!sleep_param)
@@ -475,7 +798,7 @@ int pxa_suspend(void)
 	ASIC3_GPIO_INTSTAT_D = 0;
 #endif
 
-#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
 	if ( CCCR != 0x241 ) {
 	  cpu_xscale_sl_change_speed_241_without_lcd();
 	}
@@ -483,20 +806,39 @@ int pxa_suspend(void)
 	DPRINTK("FCS : CCCR = %x\n",cccr_reg);
 #endif
 
-	    
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	// check jacket status.
+	sharpsl_request_dac_init();     // in sharpsl_battery.c
+	if(sharpsl_battery_charge_hook)
+	  sharpsl_battery_charge_hook(4); // check jacket.
+#endif
+
 DO_SUSPEND:
-#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI)
-
 
+#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
+	DPRINTK("pass_charge_flag = %d\n",pass_charge_flag);
 	if ( !pass_charge_flag ) {
 	  // not charging and AC-IN !
-	  if ( !charge_status && ( GPLR(GPIO_AC_IN) & GPIO_bit(GPIO_AC_IN)) != 0 ) {
+
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	  DPRINTK("check AC-adaptor\n");
+	  if ( !charge_status && ( GPLR(GPIO_AC_IN) & GPIO_bit(GPIO_AC_IN) ) == 0){
+#else
+	  if ( !charge_status && ( GPLR(GPIO_AC_IN) & GPIO_bit(GPIO_AC_IN)) != 0){
+#endif
 	    DPRINTK("kick charging\n");
 	    charge_status = 1;
 	    sharpsl_off_charge_battery();
 	  }
 	}
 
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	  tosa_check_ac_and_jacket_state();
+	  DPRINTK("ac97&tc6393 down\n");
+	  tc6393_suspend();
+	  tosa_ac97_exit();
+#endif
+
 	if ( charge_status ) {
 	  if (  ( ( RTAR - RCNR ) < ( BATTERY_CHECK_TIME + 30 ) ) && ( RTSR & RTSR_ALE )  ) {
 	    // maybe alarm will occur
@@ -533,6 +875,8 @@ DO_SUSPEND:
 	} else {
 	  PGSR1 &= ~GPIO_bit(GPIO43_BTTXD);
 	}
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+// non
 #endif
 
 #endif
@@ -579,12 +923,12 @@ DO_SUSPEND:
 	LCM_ICR &= ~0x100;
 #endif
 
-#if !defined(CONFIG_SABINAL_DISCOVERY)
+#if !defined(CONFIG_SABINAL_DISCOVERY) 
 	/* Scoop suspend */
 	sys_ctx.scp_gpwr = SCP_REG_GPWR;
 	SCP_REG_GPWR = 0;
-#endif
 
+#endif
 	sys_ctx.gpdr0 = GPDR0;
 	sys_ctx.gpdr1 = GPDR1;
 	sys_ctx.gpdr2 = GPDR2;
@@ -640,7 +984,9 @@ DO_SUSPEND:
 		PWER = WAKEUP_SRC & apm_wakeup_src_mask;
 		PRER = R_WAKEUP_SRC & apm_wakeup_src_mask;
 		PFER = F_WAKEUP_SRC & apm_wakeup_src_mask;
+
 		PEDR = WAKEUP_SRC & apm_wakeup_src_mask;
+
 		for (i = 0; i <=15; i++) {
 			if ( PRER & PFER & GPIO_bit(i) ) {
 				if ( GPLR0 & GPIO_bit(i) )
@@ -650,20 +996,31 @@ DO_SUSPEND:
 			}
 		}
 
-
 		/* Clear reset status */
 		RCSR = RCSR_HWR | RCSR_WDR | RCSR_SMR | RCSR_GPR;
 
 		/* Stop 3.6MHz and drive HIGH to PCMCIA and CS */
 		PCFR = PCFR_OPDE;
 
-#ifdef CONFIG_ARCH_PXA_CORGI
+#if defined(CONFIG_ARCH_PXA_CORGI)
 		/* GPIO Sleep Register */
 		PGSR2 = (PGSR2 & ~GPIO_ALL_STROBE_BIT) | GPIO_STROBE_BIT(0);
-
 		GPDR0 = 0xD3F8B040;
 		GPDR1 = 0x00FFAFC3;
 		GPDR2 = 0x0001C004;
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+		/* GPIO Sleep Register */
+		PGSR1 = (PGSR1 & ~GPIO_LOW_STROBE_BIT);
+		PGSR2 = (PGSR2 & ~GPIO_HIGH_STROBE_BIT);
+#ifdef LOGICAL_WAKEUP_SRC
+		if (logical_wakeup_src_mask &
+		    (IDPM_WAKEUP_ADDRESSBOOK|IDPM_WAKEUP_CALENDAR|IDPM_WAKEUP_MENU|IDPM_WAKEUP_MAIL)) {
+		  PGSR1 |= GPIO_STROBE_BIT(4);
+		}
+#endif
+		GPDR0 = 0xC3810940;
+		GPDR1 = 0xFCFFAB82;
+		GPDR2 = 0x000F501f;
 #endif
 	}
 #endif
@@ -681,7 +1038,6 @@ DO_SUSPEND:
 
 #if !defined(CONFIG_SABINAL_DISCOVERY)
 	sharpsl_wakeup_check();
-
         FFMCR = sys_ctx.ffmcr;
         FFSPR = sys_ctx.ffspr;
         FFLCR = sys_ctx.fflcr;
@@ -764,21 +1120,49 @@ DO_SUSPEND:
 #endif
 
 #if !defined(CONFIG_SABINAL_DISCOVERY)
-	pxa_ssp_init();
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	if ( sharpsl_alarm_flag ) {
+	  RCNR_buffer = RCNR;
+	}
+	DPRINTK("ac97&tc6393 up\n");
+	tosa_ac97_init();
+	pxa_ac97_get(&codec,&ac97_on); // initialize 'codec' pointer
+	i2c_init(1);
+	tc6393_resume();
 
-#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI)
+	sharpsl_request_dac_init(); // in sharpsl_battery.c
+#else
+	pxa_ssp_init();
+#endif
+
+#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
 	if ( sharpsl_alarm_flag ) {
 	  RTAR_buffer2 = RTAR;
 	  RTAR = RTAR_buffer;
 	  DPRINTK("back the ALARM Time\n");
 	}
 
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	if ( sharpsl_alarm_flag && ( RCNR_buffer == RTAR_buffer2 ) ) {
+	  //printk("RCNR_buffer=%d\n",RCNR_buffer);
+	  if(sharpsl_battery_charge_hook)
+	    sharpsl_battery_charge_hook(5);
+#if 1
+	  if(tosa_check_charge_full(BATTERY_CHECK_TIME) < 0) {
+	    goto DO_SUSPEND;
+	  }
+#else
+	  goto DO_SUSPEND;
+#endif
+	}
+#else
 	if ( sharpsl_alarm_flag && ( RCNR == RTAR_buffer2 ) ) {
 	  if ( sharpsl_off_charge_battery() ) {
 	    DPRINTK("charge timer \n");
 	    goto DO_SUSPEND;
 	  }
 	}
+#endif
 #endif
 
 	/* ----- hardware resume ----- */
@@ -788,6 +1172,26 @@ DO_SUSPEND:
 	}
 
 #if !defined(CONFIG_POODLE_TR0) && !defined(CONFIG_CORGI_TR0)
+#if defined(CONFIG_ARCH_PXA_SHEPHERD)
+	if ( ( (GPLR(GPIO_MAIN_BAT_LOW) & GPIO_bit(GPIO_MAIN_BAT_LOW)) == 0 )
+	     || ( !sharpsl_off_mode && chkFatalBatt() == 0 ) ) {
+	        printk("return to suspend (fatal) ....\n");
+		goto DO_SUSPEND;
+	}
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+	if ( (GPLR(GPIO_AC_IN) & GPIO_bit(GPIO_AC_IN)) == 0){
+	  if(sharpsl_off_charge_battery() < 0){
+	    printk("return to suspend (no main batt) ....\n");
+	    goto DO_SUSPEND;
+	  }
+	}
+
+	if ( ( (GPLR(GPIO_BAT_LOCKED) & GPIO_bit(GPIO_BAT_LOCKED)) == 0 )
+	     || ( !sharpsl_off_mode && chkFatalBatt() == 0 ) ) {
+	        printk("return to suspend (fatal) ....\n");
+		goto DO_SUSPEND;
+	}
+#else
 	if ( ( (GPLR(GPIO_MAIN_BAT_LOW) & GPIO_bit(GPIO_MAIN_BAT_LOW)) == 0 )
 	     || ( chkFatalBatt() == 0 ) ) {
 	        printk("return to suspend (fatal) ....\n");
@@ -795,9 +1199,10 @@ DO_SUSPEND:
 	}
 #endif
 #endif
+#endif
 
 
-#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
 	PMCR = 0x01;
 
 	if ( sharpsl_off_mode )
@@ -805,10 +1210,15 @@ DO_SUSPEND:
 #endif
 
 
-#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
 	if ( sharpsl_chg_freq == 0x0145 ) {
 	  cpu_xscale_sl_change_speed_145_without_lcd();
 	}
+#if defined(CONFIG_ARCH_PXA_SHEPHERD) || defined(CONFIG_ARCH_PXA_TOSA)
+	else if ( sharpsl_chg_freq == 0x0161 ) {
+	  cpu_xscale_sl_change_speed_161();
+	}
+#endif
 	cccr_reg = CCCR;
 	printk("FCS : CCCR = %x\n",cccr_reg);
 
@@ -906,16 +1316,14 @@ DO_SUSPEND:
 int pm_do_suspend(void)
 {
 	int retval;
-	
-	DPRINTK("yea\n");
-	
 	retval = pm_send_all(PM_SUSPEND, (void *)2);
-	if (retval) 
+	if (retval)
 		return retval;
 
 	retval = pxa_suspend();
 
 	retval = pm_send_all(PM_RESUME, (void *)0);
+
 	if (retval)
 		return retval;
 
@@ -942,7 +1350,7 @@ static struct ctl_table pm_dir_table[] =
 
 
 
-int pxa_fatal_suspend(void)
+static int pxa_fatal_suspend(void)
 {
 	unsigned long   flags;
 	unsigned long	RTAR_buffer;
@@ -958,7 +1366,7 @@ int pxa_fatal_suspend(void)
 	save_flags_cli(flags);
 	clf();
 
-#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_POODLE) || defined(CONFIG_ARCH_PXA_CORGI)  || defined(CONFIG_ARCH_PXA_TOSA)
 	if ( CCCR != 0x241 ) {
 	  cpu_xscale_sl_change_speed_241_without_lcd();
 	}
@@ -994,6 +1402,8 @@ DO_SUSPEND:
 	  PGSR1 &= ~GPIO_bit(GPIO43_BTTXD);
 	}
 #endif
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+// non
 #endif
     
 #if defined(CONFIG_ARCH_PXA_POODLE)
@@ -1047,6 +1457,7 @@ DO_SUSPEND:
 		PRER = R_WAKEUP_SRC & apm_wakeup_src_mask;
 		PFER = F_WAKEUP_SRC & apm_wakeup_src_mask;
 		PEDR = WAKEUP_SRC & apm_wakeup_src_mask;
+
 		for (i = 0; i <=15; i++) {
 			if ( PRER & PFER & GPIO_bit(i) ) {
 				if ( GPLR0 & GPIO_bit(i) )
@@ -1056,20 +1467,32 @@ DO_SUSPEND:
 			}
 		}
 
-
 		/* Clear reset status */
 		RCSR = RCSR_HWR | RCSR_WDR | RCSR_SMR | RCSR_GPR;
 
 		/* Stop 3.6MHz and drive HIGH to PCMCIA and CS */
 		PCFR = PCFR_OPDE;
 
-#ifdef CONFIG_ARCH_PXA_CORGI
+
+#if defined(CONFIG_ARCH_PXA_CORGI)
 		/* GPIO Sleep Register */
 		PGSR2 = (PGSR2 & ~GPIO_ALL_STROBE_BIT) | GPIO_STROBE_BIT(0);
-
 		GPDR0 = 0xD3F8B040;
 		GPDR1 = 0x00FFAFC3;
 		GPDR2 = 0x0001C004;
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+		/* GPIO Sleep Register */
+		PGSR1 = (PGSR1 & ~GPIO_LOW_STROBE_BIT);
+		PGSR2 = (PGSR2 & ~GPIO_HIGH_STROBE_BIT);
+#ifdef LOGICAL_WAKEUP_SRC
+		if (logical_wakeup_src_mask &
+		    (IDPM_WAKEUP_ADDRESSBOOK|IDPM_WAKEUP_CALENDAR|IDPM_WAKEUP_MENU|IDPM_WAKEUP_MAIL)) {
+		  PGSR1 |= GPIO_STROBE_BIT(4);
+		}
+#endif
+		GPDR0 = 0xC3810940;
+		GPDR1 = 0xFCFFAB82;
+		GPDR2 = 0x000F501f;
 #endif
 	}
 #endif
@@ -1126,13 +1549,19 @@ void sharpsl_poodle_fataloff(void)
 }
 #endif
 
-#if defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
 #define	SCP_INIT_DATA(adr,dat)	(((adr)<<16)|(dat))
 #define	SCP_INIT_DATA_END	((unsigned long)-1)
 void sharpsl_corgi_fataloff(void)
 {
 #ifdef CONFIG_PM
+#if defined(CONFIG_ARCH_PXA_TOSA)
+  tc6393_fatal_off();
+  tc6393_suspend();
+  tosa_ac97_exit();
+#else
   w100_fatal_off();
+#endif
 #endif
 
   {
@@ -1151,12 +1580,37 @@ void sharpsl_corgi_fataloff(void)
 		SCP_INIT_DATA(SCP_GPWR,SCP_IO_OUT),  // 24
 		SCP_INIT_DATA_END
 	};
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	static const unsigned long scp_jc_init[] =
+	{
+		SCP_INIT_DATA(SCP_MCR,0x0140),  // 00
+		SCP_INIT_DATA(SCP_MCR,0x0100),
+		SCP_INIT_DATA(SCP_CDR,0x0000),  // 04
+		SCP_INIT_DATA(SCP_CPR,0x0000),  // 0C
+		SCP_INIT_DATA(SCP_CCR,0x0000),  // 10
+		SCP_INIT_DATA(SCP_IMR,0x0000),  // 18
+		SCP_INIT_DATA(SCP_IRM,0x00FF),  // 14
+		SCP_INIT_DATA(SCP_ISR,0x0000),  // 1C
+		SCP_INIT_DATA(SCP_IRM,0x0000),
+		SCP_INIT_DATA(SCP_GPCR,SCP_JC_IO_DIR),  // 20
+		SCP_INIT_DATA(SCP_GPWR,SCP_JC_IO_OUT),  // 24
+		SCP_INIT_DATA_END
+	};
+#endif
 	int	i;
 	for(i=0; scp_init[i] != SCP_INIT_DATA_END; i++)
 	{
 		int	adr = scp_init[i] >> 16;
 		SCP_REG(adr) = scp_init[i] & 0xFFFF;
 	}
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	for(i=0; scp_jc_init[i] != SCP_INIT_DATA_END; i++)
+	{
+		int	adr = scp_jc_init[i] >> 16;
+		SCP_JC_REG(adr) = scp_jc_init[i] & 0xFFFF;
+	}
+#endif
+
   }
 
 }
@@ -1165,6 +1619,10 @@ void sharpsl_corgi_fataloff(void)
 
 void sharpsl_fataloff(void)
 {
+#if defined(CONFIG_ARCH_PXA_TOSA)
+    return;
+#endif
+
 	if ( PSSR & 0x02 ) {
 
 	  printk("PSSR = %8x\n",PSSR);
@@ -1176,19 +1634,18 @@ void sharpsl_fataloff(void)
 	  PSPR = 0x00;
 	  RCSR = 0xf;
 
-	  printk("off\n");
 	  pxa_fatal_suspend();
-	  printk("off 2\n");
 #endif
-#if defined(CONFIG_ARCH_PXA_CORGI)
+#if defined(CONFIG_ARCH_PXA_CORGI) || defined(CONFIG_ARCH_PXA_TOSA)
 	  sharpsl_corgi_fataloff();
 
 	  PSPR = 0x00;
 	  RCSR = 0xf;
 
-	  printk("off\n");
 	  pxa_fatal_suspend();
-	  printk("off 2\n");
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	  sharpsl_restart();
+#endif
 #endif
 	}
 }
@@ -1206,12 +1663,14 @@ static int sharpsl_off_thread(void* unused)
 		interruptible_sleep_on(&wq_off);
 		DPRINTK("start off sequence ! \n");
 
+#if !defined(CONFIG_ARCH_PXA_SHEPHERD) && !defined(CONFIG_ARCH_PXA_TOSA)
 		sharpsl_off_mode = 1;
-
+#endif
 		handle_scancode(SLKEY_OFF|KBDOWN , 1);
 		mdelay(10);
 		handle_scancode(SLKEY_OFF|KBUP   , 0);
 
+#if !defined(CONFIG_ARCH_PXA_SHEPHERD) && !defined(CONFIG_ARCH_PXA_TOSA)
 		// wait off signal
 		// if can not recieve off siganl , so force off.
 		time_cnt = jiffies;
@@ -1223,27 +1682,52 @@ static int sharpsl_off_thread(void* unused)
 		// maybe apps is dead, so we have to restart.
 		pm_do_suspend();
 		sharpsl_restart();
+#endif
 	}
 	return 0;
 }
 
+#if defined(CONFIG_ARCH_PXA_SHEPHERD) || defined(CONFIG_ARCH_PXA_TOSA)
+static struct timer_list bat_switch_timer;
+
+void sharpsl_emerge_restart(void)
+{
+	unsigned long   flags;
+	save_flags_cli(flags);
+	sharpsl_restart();
+}
+#endif
 
 void sharpsl_emerge_off(int irq, void *dev_id, struct pt_regs *regs)
 {
 
+  DPRINTK("DBG:sharpsl_emerge_off [non-battery]\n");
+
   /* noise ? */
   mdelay(10);
-  if ( 0x1234ABCD != regs &&
-       GPLR(GPIO_MAIN_BAT_LOW) & GPIO_bit(GPIO_MAIN_BAT_LOW) ) {
+#if defined(CONFIG_ARCH_PXA_TOSA)
+  if ( 0x1234ABCD != regs && GPLR(GPIO_BAT_LOCKED) & GPIO_bit(GPIO_BAT_LOCKED) )
+#else
+  if ( 0x1234ABCD != regs && GPLR(GPIO_MAIN_BAT_LOW) & GPIO_bit(GPIO_MAIN_BAT_LOW) )
+#endif
+    {
+#if defined(CONFIG_ARCH_PXA_SHEPHERD)
+	  sharpsl_battery_charge_hook(1);	/* charge off */
+	  if (sharppda_kbd_resetcheck()) {
+		  sharpsl_off_mode = 2;			/* off in maintenance */
+		  mod_timer(&bat_switch_timer, jiffies + HZ * 5);
+		  return;
+	  }
+#else
     printk("cancel emergency off\n");
+#endif
     return;
   }
 
-#if defined(CONFIG_ARCH_PXA_CORGI)
+#if (defined(CONFIG_ARCH_PXA_CORGI)) && !defined(CONFIG_ARCH_PXA_SHEPHERD) && !defined(CONFIG_ARCH_PXA_TOSA)
   if (!(GPLR(GPIO_MAIN_BAT_LOW) & GPIO_bit(GPIO_MAIN_BAT_LOW)))
     apm_wakeup_src_mask = 0;
 #endif
-
   wake_up(&wq_off);
 
 }
@@ -1259,22 +1743,41 @@ static int __init pm_init(void)
 
 #ifndef CONFIG_SABINAL_DISCOVERY
 	/* Set transition detect */
+#ifdef CONFIG_ARCH_PXA_SHEPHERD
+	set_GPIO_IRQ_edge( GPIO_MAIN_BAT_LOW  , GPIO_BOTH_EDGES );
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+	set_GPIO_IRQ_edge( GPIO_BAT_LOCKED  , GPIO_BOTH_EDGES );
+#else
 	set_GPIO_IRQ_edge( GPIO_MAIN_BAT_LOW  , GPIO_FALLING_EDGE );
+#endif
 
 
 	/* this registration can be done in init/main.c. */
 	if(1){
 	  int err;
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	  err = request_irq(IRQ_GPIO_BAT_LOCKED,sharpsl_emerge_off , SA_INTERRUPT, "batok", NULL);
+#else
 	  err = request_irq(IRQ_GPIO_MAIN_BAT_LOW,sharpsl_emerge_off , SA_INTERRUPT, "batok", NULL);
+#endif
 	  if( err ){
 	    printk("batok install error %d\n",err);
 	  }else{
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	    enable_irq(IRQ_GPIO_BAT_LOCKED);
+#else
 	    enable_irq(IRQ_GPIO_MAIN_BAT_LOW);
+#endif
 	    printk("batok installed\n");
 	  }
 	}
 
 	kernel_thread(sharpsl_off_thread,  NULL, CLONE_FS | CLONE_FILES | CLONE_SIGHAND | SIGCHLD);
+#if defined(CONFIG_ARCH_PXA_SHEPHERD) || defined(CONFIG_ARCH_PXA_TOSA)
+//#if defined(CONFIG_ARCH_PXA_SHEPHERD) 
+	init_timer(&bat_switch_timer);
+	bat_switch_timer.function = sharpsl_emerge_restart;
+#endif
 #endif
 	return 0;
 }
@@ -1331,12 +1834,55 @@ void PrintParamTable_P(void) {
 
 void set_apm_wakeup_src_mask(u32 SetValue)
 {
+#ifdef LOGICAL_WAKEUP_SRC
+	logical_wakeup_src_mask = 0;
+	// RTC
+	if (SetValue&IDPM_WAKEUP_RTC) {
+	  apm_wakeup_src_mask |= GPIO_bit(31);
+	  logical_wakeup_src_mask |= IDPM_WAKEUP_RTC;
+	} else apm_wakeup_src_mask &= ~GPIO_bit(31);
+#if defined(CONFIG_ARCH_PXA_TOSA)
+	// Sync key
+	if (SetValue&IDPM_WAKEUP_SYNC) {
+	  apm_wakeup_src_mask |= GPIO_bit(GPIO_SYNC);
+	  logical_wakeup_src_mask |= IDPM_WAKEUP_SYNC;
+	} else apm_wakeup_src_mask &= ~GPIO_bit(GPIO_SYNC);
+	// Record key
+	if (SetValue&IDPM_WAKEUP_REC) {
+	  apm_wakeup_src_mask |= GPIO_bit(GPIO_RECORD_BTN);
+	  logical_wakeup_src_mask |= IDPM_WAKEUP_REC; 
+	} else apm_wakeup_src_mask &= ~GPIO_bit(GPIO_RECORD_BTN);
+	// USBD
+	if (SetValue&IDPM_WAKEUP_USBD) {
+	  apm_wakeup_src_mask |= GPIO_bit(GPIO_USB_IN);
+	  logical_wakeup_src_mask |= IDPM_WAKEUP_USBD;
+	} else apm_wakeup_src_mask &= ~GPIO_bit(GPIO_USB_IN);
+	// only Logical
+	if (SetValue&IDPM_WAKEUP_AC)		logical_wakeup_src_mask |= IDPM_WAKEUP_AC;
+	if (SetValue&IDPM_WAKEUP_JACKET)	logical_wakeup_src_mask |= IDPM_WAKEUP_JACKET;
+	if (SetValue&IDPM_WAKEUP_CALENDAR)	logical_wakeup_src_mask |= IDPM_WAKEUP_CALENDAR;
+	if (SetValue&IDPM_WAKEUP_ADDRESSBOOK)	logical_wakeup_src_mask |= IDPM_WAKEUP_ADDRESSBOOK;
+	if (SetValue&IDPM_WAKEUP_MAIL)		logical_wakeup_src_mask |= IDPM_WAKEUP_MAIL;
+	if (SetValue&IDPM_WAKEUP_MENU)		logical_wakeup_src_mask |= IDPM_WAKEUP_MENU;
+#ifdef WUPSRC_DEBUG
+	printk("set_wakeup_src=%08x->%08x[%08x]\n",SetValue,logical_wakeup_src_mask,apm_wakeup_src_mask);
+#endif
+#endif		
+#else
 	apm_wakeup_src_mask	= SetValue;
+#endif
 }
 
 u32 get_apm_wakeup_src_mask(void)
 {
+#ifdef LOGICAL_WAKEUP_SRC
+#ifdef WUPSRC_DEBUG
+  printk("get_wakeup_src=%08x\n",logical_wakeup_src_mask);
+#endif
+	return (u32)logical_wakeup_src_mask;
+#else
 	return (u32)apm_wakeup_src_mask;
+#endif
 }
 
 u32 get_apm_wakeup_factor(void)
