@@ -20,6 +20,7 @@
  * GNU General Public License for more details.
  *
  * Change Log
+ *	16-Jan-2003 SHARP sleep_on -> interruptible_sleep_on
  *
  */
 #include <linux/init.h>
@@ -100,6 +101,7 @@ enum { CORGI_HP_AUDIO=0, CORGI_HP_BUZZER, CORGI_HP_SOUNDER, CORGI_HP_HP, CORGI_H
 #define DMA_NO_INITIALIZED		1
 //#undef DMA_NO_INITIALIZED
 #define ALARM_NO_MALLOC		1
+#define BUZZER_FORCE_CLOSE		1
 
 #ifdef AUDIO_SP_DELAY_OFF
 #define AUDIO_SP_DELAY_TIME	(HZ*15)
@@ -140,6 +142,12 @@ static int corgi_intmode_step = 0;
 struct timer_list speaker_timers;
 #endif
 
+#ifdef BUZZER_FORCE_CLOSE
+#define WAIT_BZ_RELEASE	( ( 2 * 1000 ) / 10 )
+static int buzzer_open = 0;
+static int buzzer_close = 0;
+#endif
+
 /**** Prototype *******************************************************/
 static void CorgiInit(void);
 static int CorgiSetVolume(int);
@@ -1694,7 +1702,33 @@ int audio_open(struct inode *inode, struct file *file )
 	err = -EBUSY;
 
 	if ( state->wr_ref || state->rd_ref )
+#ifdef BUZZER_FORCE_CLOSE
+	  {
+	    if( !buzzer_open ){
+	      goto out;
+	    }else{
+	      int now;
+
+	      // force close buzzer
+	      PXAI2S_DBGPRINT(DBG_LEVEL1, "force a buzzer stop!\n");
+	      buzzer_close = 1;
+	      if(DCSR(os->dma_ch) & DCSR_RUN) {
+		DCSR(os->dma_ch) &=
+		  (DCSR_RUN|DCSR_ENDINTR|DCSR_STARTINTR|DCSR_BUSERR);
+	      }
+	      now = jiffies;
+	      while(1) {
+		if ( !buzzer_open ) break;
+		schedule();
+		if ( jiffies > ( now + WAIT_BZ_RELEASE ) ) break;
+	      }
+	      buzzer_close = 0;
+	      if( buzzer_open ) goto out;
+	    }
+	  }
+#else
 		goto out;
+#endif
 
 	/* request DMA channels */
 	if (file->f_mode & FMODE_WRITE) {
@@ -2031,19 +2065,19 @@ static void corgi_hp_mode(int mode)
   }
 #endif
 
-	if(( mode == CORGI_HP_AUDIO )||( mode == CORGI_HP_BUZZER )){
-		if( corgi_HPstatus < 0 ){
-			corgi_HPstatus = GPLR(GPIO_HP_IN) & GPIO_bit(GPIO_HP_IN);
+	if( corgi_HPstatus < 0 ){
+		corgi_HPstatus = GPLR(GPIO_HP_IN) & GPIO_bit(GPIO_HP_IN);
 #ifdef SOUND_SUPPORT_REMOCON
-			if( corgi_HPstatus == HPJACK_STATE_NONE ){
-				if( get_remocon_state() == HPJACK_STATE_REMOCON ){
-					corgi_HPstatus = HPJACK_STATE_REMOCON;
-				}
+		if( corgi_HPstatus == HPJACK_STATE_NONE ){
+			if( get_remocon_state() == HPJACK_STATE_REMOCON ){
+				corgi_HPstatus = HPJACK_STATE_REMOCON;
 			}
-#endif
 		}
-		state = corgi_HPstatus;
+#endif
+	}
+	state = corgi_HPstatus;
 
+	if(( mode == CORGI_HP_AUDIO )||( mode == CORGI_HP_BUZZER )){
 		if ( state ) { 	// Insert HP
 			PXAI2S_DBGPRINT(DBG_LEVEL1, "Insert HP\n");
 			mode = CORGI_HP_HP;
@@ -2069,8 +2103,13 @@ static void corgi_hp_mode(int mode)
 		reset_scoop_gpio( SCP_MUTE_L | SCP_MUTE_R | SCP_APM_ON );
 		break;
 	case CORGI_HP_HEADSET:
-		set_scoop_gpio( SCP_MIC_BIAS | SCP_MUTE_R );	//  MIC bias on
-		reset_scoop_gpio( SCP_MUTE_L | SCP_APM_ON );
+		if ( state ) { 	// Insert HP
+			set_scoop_gpio( SCP_MIC_BIAS | SCP_MUTE_R );	//  MIC bias on
+			reset_scoop_gpio( SCP_MUTE_L | SCP_APM_ON );
+		} else {	// Not Insert HP
+			set_scoop_gpio( SCP_MIC_BIAS | SCP_MUTE_R | SCP_APM_ON);
+			reset_scoop_gpio( SCP_MUTE_L );
+		}
 		break;
 	case CORGI_HP_AUDIO:
 	case CORGI_HP_BUZZER:
@@ -2397,7 +2436,7 @@ static void corgi_intmode_mix(unsigned char *dest,int bUsed)
 	  corgi_intmode = 0;
 	  int_data_org = NULL;
   }
-  return cnt;
+  return;
 }
 #else
 static void corgi_intmode_mix(unsigned char *dest,int bUsed)
@@ -2445,6 +2484,12 @@ int audio_buzzer_write(const char *buffer,int count)
   while (count > 0) {
     audio_buf_t *b = &s->buffers[s->usr_frag];
 
+#ifdef BUZZER_FORCE_CLOSE
+    if( buzzer_close ) {
+      break;
+    }
+#endif
+
     chunksize = s->fragsize - b->offset;
     if (chunksize > count)
       chunksize = count;
@@ -2496,6 +2541,10 @@ void audio_buzzer_sync(void)
   DECLARE_WAITQUEUE(wait, current);
 
 
+#ifdef BUZZER_FORCE_CLOSE
+  if( buzzer_close ) goto sync_end;
+#endif
+
   //audio_sync(file);
 
 	/*
@@ -2526,6 +2575,9 @@ void audio_buzzer_sync(void)
 #endif
 	}
 
+#ifdef BUZZER_FORCE_CLOSE
+ sync_end:;
+#endif
 
   /* Wait for DMA to complete. */
   set_current_state(TASK_INTERRUPTIBLE);
@@ -2544,7 +2596,7 @@ int audio_buzzer_release(void)
 {
   audio_state_t  *state = &pxa_audio_state;
   audio_stream_t *s = state->output_stream;
-  int err;
+  int err, ret = 0;
   DECLARE_WAITQUEUE(wait, current);
 
 
@@ -2573,12 +2625,16 @@ int audio_buzzer_release(void)
   audio_clear_buf(state->output_stream);
   *state->output_stream->drcmr = 0;
   pxa_free_dma(state->output_stream->dma_ch);
-  state->wr_ref = 0;
 
 #if ALARM_NO_MALLOC
 	// wait playing int sound.
+#ifdef BUZZER_FORCE_CLOSE
+	while(corgi_intmode_direct && !buzzer_close)
+	  schedule();
+#else
 	while(corgi_intmode_direct)
 	  schedule();
+#endif
 
 	corgi_intmode = 0;
     int_data_org = NULL;
@@ -2593,17 +2649,22 @@ int audio_buzzer_release(void)
   err = wm8731_close();
   if ( err ) {
     PXAI2S_DBGPRINT(DBG_ALWAYS, "Can not close wm8731\n");
-    return err;
+	ret = err;
   }
 
   // Close I2S
   err = i2s_close( CORGI_I2S_ADDRESS );
   if ( err ) {
     PXAI2S_DBGPRINT(DBG_ALWAYS, "Can not close i2s\n");
-    return err;
+	ret = err;
   }
 
-  return 0;
+  state->wr_ref = 0;
+#ifdef BUZZER_FORCE_CLOSE
+  buzzer_open = 0;
+#endif
+
+  return ret;
 }
 
 int audio_buzzer_open(int speed)
@@ -2611,7 +2672,7 @@ int audio_buzzer_open(int speed)
   audio_state_t  *state = &pxa_audio_state;
   audio_stream_t *os = state->output_stream;
   SOUND_SETTINGS settings;
-  int err = 0;
+  int err;
   int now;
 
   while(1) {
@@ -2619,11 +2680,16 @@ int audio_buzzer_open(int speed)
     schedule();
   }
 
+  err = -EBUSY;
   if ( state->wr_ref || state->rd_ref )
     goto out;
 
   state->wr_ref = 1;
 
+#ifdef BUZZER_FORCE_CLOSE
+  buzzer_open = 1;
+#endif
+
   /* request DMA channels */
   err = pxa_request_dma(os->name, DMA_PRIO_LOW, audio_dma_irq, os);
   if ( err < 0 )
@@ -2677,6 +2743,12 @@ int audio_buzzer_open(int speed)
   corgi_hp_mode(CORGI_HP_BUZZER);
 
   err = 0;
+#ifdef BUZZER_FORCE_CLOSE
+  if( buzzer_close ){
+    err = -EBUSY;
+    goto error;
+  }
+#endif
 out:
   return err;
 
@@ -3001,7 +3073,7 @@ static int CorgiPMCallBack(struct pm_dev *pm_dev,pm_request_t req, void *data)
 
 	    if ( pxa_audio_state.rd_ref || pxa_audio_state.wr_ref ) {
 	      wm8731_resume();
-	      err = i2s_open( CORGI_I2S_ADDRESS , CORGI_DEFAULT_FREQUENCY);
+	      err = i2s_open( CORGI_I2S_ADDRESS , sound.hw_freq );
 	      if ( err ) {
 		PXAI2S_DBGPRINT(DBG_ALWAYS, "Can not open i2s\n");
 	      }
@@ -3022,6 +3094,9 @@ static int CorgiPMCallBack(struct pm_dev *pm_dev,pm_request_t req, void *data)
 		  PXAI2S_DBGPRINT(DBG_LEVEL1, "status changed to %d\n", corgi_HPstatus);
 		}
 #endif
+	    if ( pxa_audio_state.rd_ref || pxa_audio_state.wr_ref ) {
+	      wake_up(&hp_proc);
+	    }
 	  break;
 	  }
 	}
