@@ -24,6 +24,8 @@
  *
  * ChangeLog:
  *	11-20-2002 SHARP	apply patch (fix small bugs in /proc)
+ *      1-Nov-2003 Sharp Corporation   for Tosa
+ *
  ********************************************************************/
 
 #include <linux/config.h>
@@ -769,8 +771,7 @@ void irlmp_do_discovery(int nslots)
 		nslots = sysctl_discovery_slots = 8;
 	}
 
-
-#if 1
+#if defined(CONFIG_ARCH_SHARP_SL)
 	/*
 	 * Clear the passive discovery retry flag.
 	 * modified by SHARP
